package info.textgrid.middleware;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import org.elasticsearch.search.aggregations.bucket.filter.Filter;
import org.elasticsearch.action.get.GetRequest;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.Strings;

import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.script.Script;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.aggregations.Aggregation;
import org.elasticsearch.search.aggregations.AggregationBuilder;
import org.elasticsearch.search.aggregations.AggregationBuilders;

import org.elasticsearch.search.aggregations.bucket.terms.Terms;
import org.elasticsearch.search.aggregations.bucket.terms.Terms.Bucket;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;

import info.textgrid.clients.tgauth.AuthClientException;
import info.textgrid.middleware.oaipmh.ListSetsType;
import info.textgrid.middleware.oaipmh.SetType;
import info.textgrid.namespaces.middleware.tgcrud.common.TextGridMimetypes;


/**
 *
 */
public class SetDeliverer {

  //private static Map<String, String> setSet = new Hashtable<String, String>();

  private Set<String> identifier = new LinkedHashSet<String>();
  private String formatField;
  private String formatToFilter;
  private String identifierField;
  private String repositoryObjectURIPrefix;
  private String specField;
  private String specFieldPrefix;
  private boolean textgrid;
  private boolean dariah;

  /**
   * @param oaiEsClient
   * @param textgrid
   * @param dariah
   */
  public SetDeliverer(boolean textgrid, boolean dariah) {
    this.textgrid = textgrid;
    this.dariah = dariah;
  }

  /**
   * @return
 * @throws AuthClientException 
 * @throws IOException 
   */
  public ListSetsType setListBuilder() {

    ListSetsType setList = new ListSetsType();
    //TODO looking for DARIAH if DARIAH works on ES6
   
    
    SearchRequest request = new SearchRequest(OAI_ESClient.getEsIndex());
    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
    AggregationBuilder agg = null;
    AggregationBuilder filterPublicProjects = null;
    AggregationBuilder projectNameAndID = null;

    
    if (this.textgrid) {    
        Script mergeProjectIDandProjectName = new Script("doc['project.id'].value + '&' +  doc['project.value.untouched'].value");
        
        filterPublicProjects = AggregationBuilders.filter("projectsPublic", QueryBuilders.boolQuery()
				.mustNot(QueryBuilders.existsQuery("nearlyPublished"))
				.must(QueryBuilders.matchPhraseQuery("format", TextGridMimetypes.EDITION)));
        
        projectNameAndID = AggregationBuilders.terms("projects")
				.script(mergeProjectIDandProjectName)
				.size(1000);   
        
        filterPublicProjects.subAggregation(projectNameAndID);

    }
    
    if (this.dariah){
    	agg = AggregationBuilders
    			.terms("descriptiveMetadata.dc:format")
    			.field("descriptiveMetadata.dc:format")
    			.size(10000);
    }
   
    List<String> fieldsForSetRequest = new ArrayList<String>();
    fieldsForSetRequest.add("project.value");
    fieldsForSetRequest.add("project.id");
    
    searchSourceBuilder.size(0);
    searchSourceBuilder.aggregation(filterPublicProjects);
    //searchSourceBuilder.aggregation(projectNameAndID);

    request.source(searchSourceBuilder);
    System.out.println(request);
    SearchResponse getRecordListItems = null;

    try {
      getRecordListItems = OAI_ESClient.getEsClient().search(request, RequestOptions.DEFAULT);      
    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }    

    if (this.textgrid == true && getRecordListItems.getAggregations()!=null) {
    
    	String projectName="";    	
      
  	  	for (Entry<String, Aggregation> entry : getRecordListItems.getAggregations().asMap().entrySet()) {
  		  String name = entry.getKey(); 
  		  
            if (name.equals("projectsPublic")) {
                
                Filter trytry = getRecordListItems.getAggregations().get("projectsPublic");
                Terms trytry2 = trytry.getAggregations().get("projects");               
                for(Bucket bentry : trytry2.getBuckets()) {
                  	projectName = bentry.getKey().toString();
                	String [] projectInfos = projectName.split("&");
                	
                	  SetType setsForTextGrid = new SetType();
                  	  setsForTextGrid.setSetSpec("project:" + projectInfos[0]);
                  	  setsForTextGrid.setSetName(projectInfos[1]);
                      setList.getSet().add(setsForTextGrid);
                	
                }
            }           
  	  }
    
    for (SearchHit hit : getRecordListItems.getHits().getHits()) {

      if (this.dariah == true && hit.getSourceAsMap().get(this.identifierField)
          .toString().startsWith("hdl:")) {
    	  
        String pid = hit.getSourceAsMap().get(this.identifierField).toString();
        this.identifier.add(pid);

      }
        
      }
    }
    
    if (this.dariah == true) {
      for (String identifierSetSpec : this.identifier) {
        String id = identifierSetSpec;
        try {
          id = URLDecoder.decode(id, "UTF-8");
        } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
        }

        id = identifierSetSpec.substring(this.repositoryObjectURIPrefix.length());

        String[] includes2 = new String[] {this.specField};
        String[] excludes2 = Strings.EMPTY_ARRAY;
        FetchSourceContext fetchSourceContext2 =
            new FetchSourceContext(true, includes2, excludes2);

        GetRequest getRequest =
            new GetRequest(OAI_ESClient.getEsIndex(), OAI_ESClient.getEsType(), id)
                .fetchSourceContext(fetchSourceContext2);
        GetResponse setSpec = null;

        try {
          setSpec = OAI_ESClient.getEsClient().get(getRequest, RequestOptions.DEFAULT);
        } catch (IOException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }

        String setName = "";
        if (setSpec != null && setSpec.isExists()) {
          if (setSpec.getField(this.specField) != null) {
            setName = setSpec.getSourceAsMap().get(this.specField).toString();
          }
        }

        SetType set = new SetType();
        set.setSetName(setName);
        set.setSetSpec(this.specFieldPrefix + id);
        setList.getSet().add(set);
      }
    }

    this.identifier.clear();

    return setList;
  }

  /**
   * @return
   */
  public String getFormatField() {
    return this.formatField;
  }

  /**
   * @param formatField
   */
  public void setFormatField(String formatField) {
    this.formatField = formatField;
  }

  /**
   * @return
   */
  public String getFormatToFilter() {
    return this.formatToFilter;
  }

  /**
   * @param formatToFilter
   */
  public void setFormatToFilter(String formatToFilter) {
    this.formatToFilter = formatToFilter;
  }

  /**
   * @return
   */
  public String getIdentifierField() {
    return this.identifierField;
  }

  /**
   * @param identifierField
   */
  public void setIdentifierField(String identifierField) {
    this.identifierField = identifierField;
  }

  /**
   * @return
   */
  public String getRepositoryObjectURIPrefix() {
    return this.repositoryObjectURIPrefix;
  }

  /**
   * @param repositoryObjectURIPrefix
   */
  public void setRepositoryObjectURIPrefix(String repositoryObjectURIPrefix) {
    this.repositoryObjectURIPrefix = repositoryObjectURIPrefix;
  }

  /**
   * @return
   */
  public String getSpecField() {
    return this.specField;
  }

  /**
   * @param titleField
   */
  public void setSpecField(String titleField) {
    this.specField = titleField;
  }

  /**
   * @return
   */
  public String getSpecFieldPrefix() {
    return this.specFieldPrefix;
  }

  /**
   * @param specFieldPrefix
   */
  public void setSpecFieldPrefix(String specFieldPrefix) {
    this.specFieldPrefix = specFieldPrefix;
  }

  /**
   * @return
   */
  public boolean isTextgrid() {
    return this.textgrid;
  }

  /**
   * @param textgrid
   */
  public void setTextgrid(boolean textgrid) {
    this.textgrid = textgrid;
  }

  /**
   * @return
   */
  public boolean isDariah() {
    return this.dariah;
  }

  /**
   * @param dariah
   */
  public void setDariah(boolean dariah) {
    this.dariah = dariah;
  }

}